module Walid
  # Validate email adresses
  #
  # Validate email addresses based on RFC5321 and RFC5322 (with updates)
  #
  # Options:
  #
  # - quotes (default: false) - allow quotings in local part (i.e. lo."lol")
  # - utf (default: false) - allow UTF-8 characters in address
  # - ip (default: false) - allow IP addresses as domain part
  class EmailValidator < ActiveModel::EachValidator
    def validate_each(record, attribute, value)
      @local, _, @domain = value.rpartition('@')

      record.errors.add(attribute, 'Invalid email format.') if check_local
    end

    private

    def check_local
      @lparser ||= Parser::EmailLocalPart.new

      parsed = @lparser.parse(@local)
      parts = parsed.map(&:keys).unique

      !(options[:quotes] && parts.include?(:q))
    rescue
      false
    end
  end
end
