require 'parslet'

module Walid
  module Parser
    # Parse local part of email address
    class EmailLocalPart < Parslet::Parser
      rule(:local_part) { dot_string | quoted_string }

      rule(:dot_string) { string >> (str('.') >> string).repeat(0) }

      rule(:string) { atom | quoted_string }

      rule(:atom) { match('[\w+=-]').repeat(1).as(:a) }
      rule(:quoted_string) do
        str('"') >> q_content.repeat(0).as(:q) >> str('"')
      end

      rule(:q_content) { q_pair | q_text }

      rule(:q_text) { match('\w') | str(' ') | match('[(),:;<>@\[\]]') }
      rule(:q_pair) { str('\\') >> match(/[ \\]/) }

      root(:local_part)
    end
  end
end
